'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _axios = require('axios');

var _axios2 = _interopRequireDefault(_axios);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var INIT = 0,
    START = 1,
    SUCCESS = 2,
    FAILURE = 3,
    ERROR = 4;
var _ref = [ArrayBuffer || Object, ArrayBufferView || Object, URLSearchParams || Object],
    ArrayBuffer = _ref[0],
    ArrayBufferView = _ref[1],
    URLSearchParams = _ref[2];


var getConfig = function getConfig(props) {
    var url = props.url,
        method = props.method,
        headers = props.headers,
        params = props.params,
        data = props.data,
        timeout = props.timeout,
        auth = props.auth,
        responseType = props.responseType,
        xsrfCookieName = props.xsrfCookieName,
        xsrfHeaderName = props.xsrfHeaderName;


    return {
        url: url, method: method, headers: headers, params: params,
        data: data, timeout: timeout, auth: auth, responseType: responseType,
        xsrfCookieName: xsrfCookieName, xsrfHeaderName: xsrfHeaderName
    };
};

var Request = function (_React$Component) {
    (0, _inherits3.default)(Request, _React$Component);

    function Request(props) {
        (0, _classCallCheck3.default)(this, Request);

        var _this = (0, _possibleConstructorReturn3.default)(this, (Request.__proto__ || (0, _getPrototypeOf2.default)(Request)).call(this, props));

        _this.onSuccess = function () {
            _this.props.onSuccess(_this.state.response);
        };

        _this.onFailure = function () {
            _this.props.onFailure(_this.state.response);
        };

        _this.onError = function () {
            _this.props.onError(_this.state.error);
        };

        _this.request = function () {
            var config = getConfig(_this.props);

            // remove the undefined keys
            (0, _keys2.default)(config).map(function (k) {
                return config[k] === undefined ? delete config[k] : false;
            });

            _this.setState({ status: START }, _this.props.onStart);
            _axios2.default.request(config).then(function (response) {
                var code = Math.round(response.status / 100);
                switch (code) {
                    case 2:
                        _this.setState({ status: SUCCESS, response: response }, _this.props.onSuccess ? _this.onSuccess : undefined);
                        break;
                    default:
                        _this.setState({ status: FAILURE, response: response }, _this.props.onFailure ? _this.onFailure : undefined);
                }
            }).catch(function (err) {
                _this.setState({ status: ERROR, error: err }, _this.props.onError ? _this.onError : undefined);
            });
        };

        _this.state = {
            status: INIT,
            response: null,
            error: null
        };
        return _this;
    }

    // noinspection JSUnusedGlobalSymbols


    (0, _createClass3.default)(Request, [{
        key: 'getChildContext',
        value: function getChildContext() {
            return (0, _extends3.default)({}, this.state, {
                request: this.request
            });
        }
    }, {
        key: 'render',
        value: function render() {
            var children = this.props.children;

            return children ? _react2.default.Children.only(children) : null;
        }

        /**
         * componentDidMount() is invoked immediately after a component is mounted.
         * Initialization that requires DOM nodes should go here.
         * If you need to load data from a remote endpoint,
         * this is a good place to instantiate the network request.
         * Setting state in this method will trigger a re-rendering.
         */

    }, {
        key: 'componentDidMount',
        value: function componentDidMount() {
            if (!this.props.defer) {
                this.request();
            }
        }

        /**
         * componentDidUpdate() is invoked immediately after updating occurs.
         * This method is not called for the initial render.
         * Use this as an opportunity to operate on the DOM when the component has been updated.
         * This is also a good place to do network requests as long as you compare
         * the current props to previous props
         * (e.g. a network request may not be necessary if the props have not changed).
         */

    }, {
        key: 'componentDidUpdate',
        value: function componentDidUpdate(prevProps) {
            // we don't need to consider the state change to determine
            // whether the component should be updated
            // since the state change is actually not reflected
            // we wanted to do all this in `shouldComponentUpdate`,
            // but we do want to update the children when state changes
            // we'll be checking if any of the props has changed the config for the axios
            // if the config has been changed only then the request should be fired

            // this should be slow as heck,
            // will change the implementation asap
            if ((0, _stringify2.default)(getConfig(this.props)) === (0, _stringify2.default)(getConfig(prevProps))) {
                return;
            }

            if (!this.props.defer) {
                this.request();
            }
        }
    }]);
    return Request;
}(_react2.default.Component);

/*
 * See: https://github.com/mzabriskie/axios#request-config
 * More PropTypes will be added as & when required.
 */


Request.propTypes = {
    url: _react2.default.PropTypes.string.isRequired,

    // the http method is not required by axios &
    // defaults to 'get' if not provided
    method: _react2.default.PropTypes.oneOf(['get', 'post', 'put', 'patch', 'delete', 'head']),

    // `headers` are custom headers to be sent
    headers: _react2.default.PropTypes.object,

    // `params` are the URL parameters to be sent with the request
    // Must be a plain object or a URLSearchParams object
    params: _react2.default.PropTypes.object,

    // `data` is the data to be sent as the request body
    // Only applicable for request methods 'PUT', 'POST', and 'PATCH'
    // When no `transformRequest` is set, must be of one of the following types:
    // - string, plain object, ArrayBuffer, ArrayBufferView, URLSearchParams
    // - Browser only: FormData, File, Blob
    // - Node only: Stream
    data: _react2.default.PropTypes.oneOfType([_react2.default.PropTypes.string, _react2.default.PropTypes.object, _react2.default.PropTypes.instanceOf(ArrayBuffer), _react2.default.PropTypes.instanceOf(ArrayBufferView), _react2.default.PropTypes.instanceOf(URLSearchParams)]),

    // `timeout` specifies the number of milliseconds before the request times out.
    // If the request takes longer than `timeout`, the request will be aborted.
    timeout: _react2.default.PropTypes.number,

    // `auth` indicates that HTTP Basic auth should be used, and supplies credentials.
    // This will set an `Authorization` header, overwriting any existing
    // `Authorization` custom headers you have set using `headers`.
    // { username: 'janedoe', password: 's00pers3cret' }
    auth: _react2.default.PropTypes.object,

    // `responseType` indicates the type of data that the server will respond with
    // options are 'arraybuffer', 'blob', 'document', 'json', 'text', 'stream'
    // default: json
    responseType: _react2.default.PropTypes.oneOf(['arraybuffer', 'blob', 'document', 'json', 'text', 'stream']),

    // `xsrfCookieName` is the name of the cookie to use as a value for xsrf token
    // default: 'XSRF-TOKEN'
    xsrfCookieName: _react2.default.PropTypes.string,

    // `xsrfHeaderName` is the name of the http header that carries the xsrf token value
    // default: 'X-XSRF-TOKEN'
    xsrfHeaderName: _react2.default.PropTypes.string,

    /*
     * Props that behave either as hooks/callbacks,
     * these are the APIs provided by us.
     */
    onStart: _react2.default.PropTypes.func,
    onSuccess: _react2.default.PropTypes.func,
    onFailure: _react2.default.PropTypes.func,
    onError: _react2.default.PropTypes.func,

    defer: _react2.default.PropTypes.bool
};

Request.childContextTypes = {
    status: _react2.default.PropTypes.oneOf([INIT, START, SUCCESS, FAILURE, ERROR]),
    response: _react2.default.PropTypes.object,
    error: _react2.default.PropTypes.object,
    request: _react2.default.PropTypes.func
};

exports.default = Request;

var RequestInit = function (_React$Component2) {
    (0, _inherits3.default)(RequestInit, _React$Component2);

    function RequestInit() {
        (0, _classCallCheck3.default)(this, RequestInit);
        return (0, _possibleConstructorReturn3.default)(this, (RequestInit.__proto__ || (0, _getPrototypeOf2.default)(RequestInit)).apply(this, arguments));
    }

    (0, _createClass3.default)(RequestInit, [{
        key: 'render',
        value: function render() {
            if (this.context.status !== INIT) {
                return null;
            }

            var children = this.props.children;

            return children ? _react2.default.Children.only(children) : null;
        }
    }]);
    return RequestInit;
}(_react2.default.Component);

RequestInit.contextTypes = {
    status: _react2.default.PropTypes.oneOf([INIT, START, SUCCESS, FAILURE, ERROR])
};

var RequestStart = function (_React$Component3) {
    (0, _inherits3.default)(RequestStart, _React$Component3);

    function RequestStart() {
        (0, _classCallCheck3.default)(this, RequestStart);
        return (0, _possibleConstructorReturn3.default)(this, (RequestStart.__proto__ || (0, _getPrototypeOf2.default)(RequestStart)).apply(this, arguments));
    }

    (0, _createClass3.default)(RequestStart, [{
        key: 'render',
        value: function render() {
            if (this.context.status !== START) {
                return null;
            }

            var children = this.props.children;

            return children ? _react2.default.Children.only(children) : null;
        }
    }]);
    return RequestStart;
}(_react2.default.Component);

RequestStart.contextTypes = {
    status: _react2.default.PropTypes.oneOf([INIT, START, SUCCESS, FAILURE, ERROR])
};

var RequestSuccess = function (_React$Component4) {
    (0, _inherits3.default)(RequestSuccess, _React$Component4);

    function RequestSuccess() {
        (0, _classCallCheck3.default)(this, RequestSuccess);
        return (0, _possibleConstructorReturn3.default)(this, (RequestSuccess.__proto__ || (0, _getPrototypeOf2.default)(RequestSuccess)).apply(this, arguments));
    }

    (0, _createClass3.default)(RequestSuccess, [{
        key: 'render',
        value: function render() {
            if (this.context.status !== SUCCESS) {
                return null;
            }

            var children = this.props.children;

            return children ? _react2.default.Children.only(children) : null;
        }
    }]);
    return RequestSuccess;
}(_react2.default.Component);

RequestSuccess.contextTypes = {
    status: _react2.default.PropTypes.oneOf([INIT, START, SUCCESS, FAILURE, ERROR]),
    response: _react2.default.PropTypes.object
};

var RequestFailure = function (_React$Component5) {
    (0, _inherits3.default)(RequestFailure, _React$Component5);

    function RequestFailure() {
        (0, _classCallCheck3.default)(this, RequestFailure);
        return (0, _possibleConstructorReturn3.default)(this, (RequestFailure.__proto__ || (0, _getPrototypeOf2.default)(RequestFailure)).apply(this, arguments));
    }

    (0, _createClass3.default)(RequestFailure, [{
        key: 'render',
        value: function render() {
            if (this.context.status !== FAILURE) {
                return null;
            }

            var children = this.props.children;

            return children ? _react2.default.Children.only(children) : null;
        }
    }]);
    return RequestFailure;
}(_react2.default.Component);

RequestFailure.contextTypes = {
    status: _react2.default.PropTypes.oneOf([INIT, START, SUCCESS, FAILURE, ERROR]),
    error: _react2.default.PropTypes.object
};

var RequestError = function (_React$Component6) {
    (0, _inherits3.default)(RequestError, _React$Component6);

    function RequestError() {
        (0, _classCallCheck3.default)(this, RequestError);
        return (0, _possibleConstructorReturn3.default)(this, (RequestError.__proto__ || (0, _getPrototypeOf2.default)(RequestError)).apply(this, arguments));
    }

    (0, _createClass3.default)(RequestError, [{
        key: 'render',
        value: function render() {
            if (this.context.status !== ERROR) {
                return null;
            }

            var children = this.props.children;

            return children ? _react2.default.Children.only(children) : null;
        }
    }]);
    return RequestError;
}(_react2.default.Component);

RequestError.contextTypes = {
    status: _react2.default.PropTypes.oneOf([INIT, START, SUCCESS, FAILURE, ERROR]),
    error: _react2.default.PropTypes.object
};

/*
 * TODO:
 * Ideally whenever the component is rendered,
 * it should set the parent Request component
 * defer prop to true
 */

var RequestDefer = function (_React$Component7) {
    (0, _inherits3.default)(RequestDefer, _React$Component7);

    function RequestDefer() {
        (0, _classCallCheck3.default)(this, RequestDefer);
        return (0, _possibleConstructorReturn3.default)(this, (RequestDefer.__proto__ || (0, _getPrototypeOf2.default)(RequestDefer)).apply(this, arguments));
    }

    (0, _createClass3.default)(RequestDefer, [{
        key: 'render',
        value: function render() {
            var _props = this.props,
                children = _props.children,
                action = _props.action;


            var props = (0, _defineProperty3.default)({}, action, this.context.request);
            return children ? _react2.default.cloneElement(_react2.default.Children.only(children), props) : null;
        }
    }]);
    return RequestDefer;
}(_react2.default.Component);

RequestDefer.propTypes = {
    action: _react2.default.PropTypes.string.isRequired
};

RequestDefer.contextTypes = {
    request: _react2.default.PropTypes.func
};

Request.Init = RequestInit;
Request.Start = RequestStart;
Request.Success = RequestSuccess;
Request.Failure = RequestFailure;
Request.Error = RequestError;
Request.Defer = RequestDefer;